#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

# This file should contain all the record creation needed to seed the database with its default values.
# The data can then be loaded with the rake db:seed (or created alongside the db with db:setup).
#
# Examples:
#
#   cities = City.create([{ name: 'Chicago' }, { name: 'Copenhagen' }])
#   Mayor.create(name: 'Emanuel', city: cities.first)

#Load Default Sensor (Root)
#unless ENV["RAILS_ENV"] && ENV["RAILS_ENV"] == "test"

if ENV["STUB_SENSORS"] != "1"
  sensor_root = Sensor.create_with(uuid: rand(1..9_223_372_036_854_775_807))
                  .find_or_create_by(name: 'root', type: 1, domain_type: 0)
  user_sensor_id = Sensor.root.id
else
  sensor_root = Sensor.new(id: 1)
  user_sensor_id = 1
end

# Default admin user setup
if User.all.blank?
  User.create(
    name: 'Administrator',
    login: RedBorder::USER_CONFIG['login'],
    email: RedBorder::USER_CONFIG['email'],
    password: RedBorder::USER_CONFIG['password'],
    password_confirmation: RedBorder::USER_CONFIG['password'],
    authentication_token: RedBorder::USER_CONFIG['authtoken'],
    admin: true,
    sensor_id: user_sensor_id,
    pwd_changed_at: Time.now
  )
end

# Default settings
Setting.find_or_create_by(name: 'company', value: 'redBorder Networks')
Setting.find_or_create_by(name: 'email', value: RedBorder::USER_CONFIG['email'])
Setting.find_or_create_by(name: 'rsa', value: nil)
Setting.find_or_create_by(name: 'minimum_password_length', value: '12')
Setting.find_or_create_by(name: 'session_timeout_time', value: '1800')
Setting.find_or_create_by(name: 'autoreload_time', value: '300')
Setting.find_or_create_by(name: 'autoreload_raw_time', value: '300')
Setting.find_or_create_by(name: 'geoip_country', value: 'https://dl.miyuru.lk/geoip/maxmind/country/maxmind4.dat.gz')
Setting.find_or_create_by(name: 'geoip_country_date', value: 'N/A')
Setting.find_or_create_by(name: 'geoip_country_custom', value: '0')
Setting.find_or_create_by(name: 'geoip_country_custom_file', value: nil)
Setting.find_or_create_by(name: 'geoip_country_md5', value: nil)
Setting.find_or_create_by(name: 'geoip_countryv6', value: 'https://dl.miyuru.lk/geoip/maxmind/country/maxmind6.dat.gz')
Setting.find_or_create_by(name: 'geoip_countryv6_date', value: 'N/A')
Setting.find_or_create_by(name: 'geoip_countryv6_custom', value: '0')
Setting.find_or_create_by(name: 'geoip_countryv6_custom_file', value: nil)
Setting.find_or_create_by(name: 'geoip_countryv6_md5', value: nil)
Setting.find_or_create_by(name: 'geoip_city', value: 'https://dl.miyuru.lk/geoip/maxmind/city/maxmind4.dat.gz')
Setting.find_or_create_by(name: 'geoip_city_date', value: 'N/A')
Setting.find_or_create_by(name: 'geoip_city_custom', value: '0')
Setting.find_or_create_by(name: 'geoip_city_custom_file', value: nil)
Setting.find_or_create_by(name: 'geoip_city_md5', value: nil)
Setting.find_or_create_by(name: 'geoip_cityv6', value: 'https://dl.miyuru.lk/geoip/maxmind/city/maxmind6.dat.gz')
Setting.find_or_create_by(name: 'geoip_cityv6_date', value: 'N/A')
Setting.find_or_create_by(name: 'geoip_cityv6_custom', value: '0')
Setting.find_or_create_by(name: 'geoip_cityv6_custom_file', value: nil)
Setting.find_or_create_by(name: 'geoip_cityv6_md5', value: 'nil')
Setting.find_or_create_by(name: 'geoip_asn', value: 'https://dl.miyuru.lk/geoip/maxmind/asn/maxmind4.dat.gz')
Setting.find_or_create_by(name: 'geoip_asn_date', value: 'N/A')
Setting.find_or_create_by(name: 'geoip_asn_custom', value: '0')
Setting.find_or_create_by(name: 'geoip_asn_custom_date', value: nil)
Setting.find_or_create_by(name: 'geoip_asn_md5', value: nil)
Setting.find_or_create_by(name: 'geoip_asnv6', value: 'https://dl.miyuru.lk/geoip/maxmind/asn/maxmind6.dat.gz')
Setting.find_or_create_by(name: 'geoip_asnv6_date', value: 'N/A')
Setting.find_or_create_by(name: 'geoip_asnv6_custom', value: '0')
Setting.find_or_create_by(name: 'geoip_asnv6_custom_file', value: nil)
Setting.find_or_create_by(name: 'geoip_asnv6_md5', value: nil)
Setting.find_or_create_by(name: 'login_banner', value: nil)
Setting.find_or_create_by(name: 'cli_banner', value: nil)
Setting.find_or_create_by(name: 'nmap_url', value: 'https://nmap.org/dist/nmap-7.92-1.x86_64.rpm')

DittoCode::Exec.if 'ENTERPRISE' do
  Setting.find_or_create_by(name: 'http_key', value: nil)
  Setting.find_or_create_by(name: 'http_key_file', value: nil)
  Setting.find_or_create_by(name: 'http_pem', value: nil)
  Setting.find_or_create_by(name: 'http_pem_file', value: nil)
end
Setting.find_or_create_by(name: 'proxy', value: '0')
Setting.find_or_create_by(name: 'proxy_address', value: nil)
Setting.find_or_create_by(name: 'proxy_port', value: nil)
Setting.find_or_create_by(name: 'proxy_user', value: nil)
Setting.find_or_create_by(name: 'proxy_password', value: nil)
Setting.find_or_create_by(name: 'autoresolv', value: '1')
Setting.find_or_create_by(name: 'forecast', value: '0')
Setting.find_or_create_by(name: 'server_key', value: 'NI83Ag1MyoBWiWnaYXWl0wSX84MCdF')
Setting.find_or_create_by(name: 'gmaps_key', value: nil)
Setting.find_or_create_by(name: 'gmaps_type', value: 'api_key')
Setting.find_or_create_by(name: 'webpage_url', value: '')
Setting.find_or_create_by(name: 'vulnerability_update_periodicity', value: 'weekly')
Setting.find_or_create_by(name: 'vulnerability_update_time', value: '00:00')
Setting.find_or_create_by(name: 'time_series_model', value: 'DoubleExponentialSmoothingModel')
Setting.find_or_create_by(name: 'anomaly_model', value: 'DBScanModel')
Setting.find_or_create_by(name: 'ksgima_threshold', value: '3')
Setting.find_or_create_by(name: 'outliers_min_cycles', value: 1)
Setting.find_or_create_by(name: 'outliers_max_cycles', value: 15)
Setting.find_or_create_by(name: 'outliers_min_intervals', value: 100)
Setting.find_or_create_by(name: 'outliers_max_intervals', value: 1500)

# Default dashboards
user = User.find_by(login: 'admin', admin: true) or User.first

# Load Default Severities
Severity.find_or_create_by(sig_id: 14, name: 'critical', text_color: "#ffffff", bg_color: "#ff4643")
Severity.find_or_create_by(sig_id: 15, name: 'extremely high', text_color: "#ffffff", bg_color: "#ff4643")
Severity.find_or_create_by(sig_id: 13, name: 'very high', text_color: "#ffffff", bg_color: "#d24643")
Severity.find_or_create_by(sig_id: 1, name: 'high', text_color: "#ffffff", bg_color: "#aa4643")
Severity.find_or_create_by(sig_id: 2, name: 'medium', text_color: "#ffffff", bg_color: "#d9c03e")
Severity.find_or_create_by(sig_id: 3, name: 'low', text_color: "#ffffff", bg_color: "#89a54e")
Severity.find_or_create_by(sig_id: 4, name: 'very low', text_color: "#ffffff", bg_color: "#30BF93")
Severity.find_or_create_by(sig_id: 11, name: 'unknown', text_color: "#ffffff", bg_color: "#919191")
Severity.find_or_create_by(sig_id: 12, name: 'clean', text_color: "#ffffff", bg_color: "#5eca5e")

#Load Default Lookups
Lookup.find_or_create_by(title: "uTrace", value: "http://en.utrace.de/?query=${ip}")
Lookup.find_or_create_by(title: "Trusted Source", value: "http://www.mcafee.com/threat-intelligence/ip/default.aspx?ip=${ip}")
Lookup.find_or_create_by(title: "whois.sc", value: "http://www.whois.sc/${ip}")
Lookup.find_or_create_by(title: "Shodan", value: "https://www.shodan.io/search?query=${ip}")

# Asign sensor_root to all users with Top Domain nil
User.where(sensor_id: nil).update_all(sensor_id: sensor_root.id)
#end

DittoCode::Exec.if 'ENTERPRISE' do
  # Tiers
  Tier.find_or_create_by(name: Tier::DEFAULT_TIER_NAME, user_id: user.id)
  Tier.find_or_create_by(name: 'hot', user_id: user.id, memory: true)
end

# Network Objects
NetObject.find_or_create_by(name: 'localhost', value: '127.0.0.1', comment: '', user_id: User.first.id)

# Protocol Objects
load(Rails.root.join('db', 'seeds', 'protocol_objects.rb'))

# Service Objects
load(Rails.root.join('db', 'seeds', 'service_objects.rb'))

# Application Objects
load(Rails.root.join('db', 'seeds', 'application_objects.rb'))
