#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class BiController < Redborder::Commons::CommonController
  # Storage of the filters
  include Redborder::Filters::Storage::Session
  # Includes
  include Redborder::Rbdruid

  self.view_paths = 'lib/modules/bi/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  helper_method :sort_column, :sort_direction

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================
  
  before_action :set_default_action, only: [
    :index, :tops, :cardinality, :clients, :comparative
  ]

  before_action :set_saved_bi_filters,
                only: [:index, :tops, :comparative,
                       :cardinality, :export_raw_to_file, :clients, :map ]
  
  before_action :set_chart_type, only: [
                        :index, :tops, :comparative, :cardinality,:load_more,
                        :clients, :export_raw_to_file, :get_raw_table, :map
                      ]

  before_action :set_range, :set_granularity,
                only: [:index, :tops, :cardinality,:load_more,
                       :export_raw_to_file, :comparative, :get_raw_table,
                       :clients, :load_clients_data, :words, :limit_words,
                       :map]

  before_action :set_aggregation, only: [
                        :index, :tops, :element_childs, :add_tab, :load_more_tops, :get_serie,
                        :load_more, :load_more_cardinality, :load_tab,
                        :get_cardinality_serie, :load_cardinality_table, :export_raw_to_file,
                        :export_cardinality_to_file, :comparative, :load_more_comparative,
                        :export_tops_to_file, :get_tops, :load_new_data, :clients,
                        :get_outliers, :info, :get_raw_table,
                        :cardinality, :load_clients_data, :add_cardinality_tab,
                        :map
                      ]
  before_action :save_aggregation_to_session, only: [
                        :index, :tops, :comparative, :element_childs, :add_tab, :load_more,
                        :load_more_tops, :load_more_comparative, :load_new_data, :get_raw_table,
                        :cardinality, :add_cardinality_tab, :clients, :load_clients_data
                      ]

  before_action :set_reports, only: [
                        :index, :tops, :cardinality, :comparative, :clients, :map
                      ]

  before_action :int_times, only: [
                        :get_serie, :get_cardinality_serie, :get_tops, :get_raw_table,
                        :load_cardinality_table, :load_more_cardinality, :load_new_data,
                        :load_more_tops, :clients, :load_clients_data,
                        :get_outliers
                      ]
  # -------------------------------------------------------------
  # Actions
  # -------------------------------------------------------------

  def export_info
    filename = in_time_machine { Time.now.strftime('%Y%m%d%H%M%S') +"-Details.json" }

    opts = params[:element].except(:timestamp, :events)
                           .delete_if { |k, v| v.blank? }
                           .map { |k, v| " -b " + Base64.urlsafe_encode64("#{k.to_s}=#{v.to_s}") }.join

    time = params[:element][:timestamp].to_time.utc
    comd = "sudo /opt/rb/bin/rb_get_raw_bi.sh -Y #{time.strftime('%Y')} "\
           "-M #{time.strftime('%m')} -D #{time.strftime('%d')} "\
           "-H #{time.strftime('%H')} -m #{time.strftime('%M')} #{opts}"

    content = `#{comd}`

    send_data content, filename: filename
  end

  #
  # => Represent zones in a map and data relative of these zones.
  #    Also represent floors.
  #    If no id is passed, the sensor is the Top Domain sensor of the
  #    current user.
  #
  def map
    # TODO: If no sensor_id is specified, top sensor must be initialized
    @sensor = if params[:id]
                Sensor.find(params[:id])
              else
                # Set top domain of the user
                current_user.sensor
              end

    # Stash filters.
    @filter.stash

    # Exclude clients without coordinates
    @filter.exclude_empty('client_latlong')

    @clients = @source.where(
      query_type: :group_by,
      start_time: @start_time,
      end_time: @end_time,
      granularity: :all,
      limit: 1000,
      group_by: [:client_name, :client_age, :client_gender, :client_country, :client_id, :client_latlong, :organization_name, :questionnaire_name],
      aggregation: :clients,
      search: @filter.get
    )

    @paths = @source.where(
          query_type: :topN,
          granularity: :minute,
          group_by: [:client_latlong],
          limit: 1000,
          aggregation: :clients,
          start_time: @start_time,
          end_time: @end_time
    )

    # Get filters
    @filter.pop
  end

  # <Clients>
  def clients
    @granularity = params[:granularity]
    @granularitymilis = GRANULARITY_TYPE[@granularity.to_sym][:step]
    @rangemilis = ((@end_time - @start_time) * 1000).to_i
    @filters_items = @filter.serialize(true)

    @top_client_age = @source.where({
        query_type: :group_by,
        granularity: :all,
        limit: 10,
        aggregation: @aggregation,
        start_time: @start_time,
        end_time: @end_time,
        group_by: [:client_age],
        sort: [@aggregation, :client_age]        
    });

    @total_client_age = @source.total({
         range: :custom,
         start_time: @start_time,
         end_time: @end_time,
         aggregation: @aggregation
     })

    @filter.remove(:type)
    @top_client_country = @source.where({
        query_type: :group_by,
        granularity: :all,
        limit: 10,
        start_time: @start_time,
        end_time: @end_time,
        group_by: [:client_country],
        aggregation: @aggregation,
        sort: [@aggregation, :client_country]
    });

    @total_client_country = @source.total({
        range: :custom,
        start_time: @start_time,
        end_time: @end_time,
        aggregation: @aggregation
    })

    @filter.remove(:type)
    @top_client_gender = @source.where({
        query_type: :group_by,
        granularity: :all,
        limit: 10,
        start_time: @start_time,
        end_time: @end_time,
        aggregation: @aggregation,
        group_by: [:client_gender],
        sort: [@aggregation, :client_gender]
    });

    @total_client_gender = @source.total({
        range: :custom,
        start_time: @start_time,
        end_time: @end_time,
        aggregation: @aggregation
    });
    @filter.remove(:type)
  end

  def load_clients_data
    @granularity = params[:granularity]
    @requested = params[:value]
    case params[:type]
    when "client_age"
      search_term = :client_age
      data = ["#{@requested}"]
    when "client_country"
      search_term = :client_country
      data = ["#{@requested}"]
    when "client_gender"
      search_term = :client_gender
      data = ["#{@requested}"]
    end

    unless search_term.nil?
      @data = @source.where(
          query_type: :timeseries,
          start_time: params[:start_time],
          end_time: params[:end_time],
          granularity: @granularity,
          aggregation: @aggregation,
          search: @filter.contains(search_term, data)
      ).map do |t|
        if @aggregation == :clients
          { event: { clients: t.clients }, timestamp: t.timestamp.to_s }
        else
          { event: { events: t.events }, timestamp: t.timestamp.to_s }
        end
      end
    else
      @data = []
    end
  end
  # </Clients

  # -------------------------------------------------------------
  # Private methods
  # -------------------------------------------------------------

  private

  #
  # Set the filters. Change current filters if params[:search] is defined.
  #
  def set_source
    @source = Redborder::DruidSource.get(source: 'bi')
    @filter = Redborder::Filter.new(@source, load_filters)
  end

  def set_saved_bi_filters
    @filters = SavedBiFilter.where(user_id: current_user.id)
    @filters_public = SavedBiFilter.where
                      .not(user_id: current_user.id).select(&:public)
  end

  #
  # Initialize the aggregation. Check to default aggregation on source.
  #
  def set_aggregation
    @aggregations = session[@source.session]
                    .fetch(:aggregations, [@source.default_aggregation])

    # See Redmine#4174
    @aggregations ||= [@source.default_aggregations]

    if params[:aggregation_action] == 'replace'
      @aggregations = [*params[:aggregation]].map(&:to_sym)
    elsif params[:aggregation_action] == 'add'
      @aggregations.unshift(params[:aggregation].to_sym)
    elsif params[:aggregation_action] == 'delete' && @aggregations.size > 1
      @aggregations.delete(params[:aggregation].to_sym)
    end

    @aggregation = if params[:aggregation] && params[:aggregation_action].nil?
                      [*params[:aggregation]].first.to_sym
                    else
                      @aggregations.first.to_sym
                    end
  end
end
