require 'test_helper'

#
# Test methods added from SensorBi to Sensor
#
class SensorBiTest < ActiveSupport::TestCase
  context 'A Bi sensor' do
    setup do
      RedBorder.send(:remove_const, 'LICENSING_MODE')
      RedBorder::LICENSING_MODE = 'global'

      @bi = sensors(:bi)
      @deployment = sensors(:deployment)
    end

    should 'be created' do
      assert @bi.valid?, @bi.errors.full_messages
    end

    should 'not be created with invalid params' do
      name_not_valid = @bi.name
      s_not_valid_name = Sensor.new(name: name_not_valid, type: @bi.type, parent: @bi.root)
      assert_not s_not_valid_name.valid?, 'Bi sensor is not valid: duplicated name.'

      #ip_not_valid = @bi.ip
      #s_not_valid_ip = Sensor.new(name: 'Bi not valid', type: @bi.type, parent: @bi.root, ip: ip_not_valid)
      #assert_not s_not_valid_ip.valid?, 'Bi sensor is not valid: duplicated ip.'
    end

    should 'be created with the same name but different type or parent' do
      name_valid = @bi.name
      s_valid = Sensor.new(name: name_valid,
                           type: @bi.type,
                           parent: @deployment,
                           ip: '192.168.104.18',
                           license: licenses(:two))
      assert s_valid.valid?, s_valid.errors.full_messages
    end

    should 'be moved into a market' do
      market = sensors(:market)

      @bi.parent_id = market.id
      assert @bi.valid?, @bi.errors.full_messages
    end

    should 'not contain children' do
      market = sensors(:market)

      market.parent_id = @bi.id
      assert_not market.valid?, 'Bi sensor can\'t contain children'
    end

    # should 'not be created with the same IP and parent' do
    #   sensor = Sensor.new(name: 'sensor_bi',
    #                       type: @bi.type,
    #                       parent: @bi.parent,
    #                       ip: @bi.ip)

    #   assert_not sensor.valid?
    # end
  end
end
