#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class LocationController < Redborder::Commons::CommonController
  # Storage of the filters
  include Redborder::Filters::Storage::Session

  # Includes
  include Redborder::Rbdruid
  include Redborder::Location::Setter
  include Redborder::Location::SessionStorageManager
  include Redborder::Location::QueryManager
  include Redborder::Location::SessionsQueryManager

  self.view_paths = 'lib/modules/location/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"
  append_view_path 'lib/modules/location/app/views/location'

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  # Common setters
  # Defined at Location::Setter
  before_action :set_session, :set_source

  before_action :store_locations, only: [:get_tops, :tops, :map, :movements, :sessions]

  before_action :set_filter,
                only: [:get_tops, :tops, :map, :movements, :load_tab, :load_serie, :sensor_children,
                       :get_serie, :sessions, :search]

  before_action :int_times, :set_times, only: [:get_serie, :load_serie]

  before_action :set_range, only: [:get_tops, :tops, :map, :movements, :load_tab, :sessions]

  before_action :set_granularity, :restore_locations,
                only: [:tops, :map, :movements, :load_tab, :load_serie, :sessions, :get_serie,
                       :get_tops]

  # Store view in session
  before_action :store_view, only: [:tops, :map, :movements, :sessions]

  # Restore variables from session
  before_action :restore_tab, :restore_view, :restore_aggregation, :restore_chart_type,
                only: [:tops, :map, :movements, :load_tab, :load_serie, :sessions, :get_tops]

  # Store aggregation in session
  before_action :store_aggregation, only: [:tops, :map, :movements, :sessions]

  # Actions For Load More
  before_action :set_load_more_options, only: :tops, if: -> { params[:load_more_offset] }

  before_action :filters_from_url, only: [:get_tops, :get_serie, :movements]
  before_action :should_skip_empty, only: [:get_serie]

  before_action :set_module_settings, only: :search
  before_action :set_saved_location_filters,
                only: [:tops, :map, :movements, :sessions]

  skip_before_action :set_aggregation, :save_aggregation_to_session,
                     :set_default_action, :set_reports, :set_chart_type,
                     :filter_params

  ## ======================================================================
  # => Methods
  ## ======================================================================

  #
  # Main Views Actions
  #

  def index
    action = { action: @session[:view] || @source.default_view }
    redirect_to action.merge(params.except(:action))
  end

  def tops
    @data = tops_data
    # Remove first N elements of data when request comes from load_more
    if @load_more_offset
      @data.shift((@load_more_offset - 1) * current_user.per_page_count)
    else
      @total = total_data
    end
  end

  def map
    @data = map_data
    @remote = true
  end

  def movements
    if @tab.to_s == "status"
      @tab = @source::TABS.except(:status).keys.first
    end
    @data = movements_data

    respond_to do |format|
      format.json { render json: @data[:chord] }
      format.html
      format.js
    end
  end

  #
  # Action for Sessions View
  #
  def sessions
    # Apply Default Filters
    @filter.add(:transition, 0)
    @filter.add(:type, 'floor')
    # Set @histogram, @percent_dwell_time, @cumulative_dwell_time, @visitors
    %w(histogram percent_dwell_time cumulative_dwell_time visitors).each do |data|
      instance_variable_set(:"@#{data}_data", send("sessions_#{data}_data"))
    end
  end

  ##
  ## Interface
  ##

  def save_tab
    @session[:tab] = params[:tab].to_sym
    render nothing: true
  end

  def load_tab
    @tab = params[:tab].to_sym
    @view = %w(tops movements total serie map).include?(params[:view]) ? params[:view] : 'tops'
    @data = send("#{@view}_data")
    @total = total_data if @view == 'tops'
  end

  def load_serie
    @tab = params[:tab].to_sym
    @uuid = params[:uuid]
    @serie = serie_data
  end

  # This method returns raw data for Overview widgets
  #
  def get_serie
    @aggregation = params[:aggregation]
    @filter.parse(params[:filters]) if params[:filters]
    @serie = if params[:w] && params[:id] != 'raw'
               @tab =
                 params[:id] == 'dot11_status' ? :status : params[:id].split('_').first.to_sym
               @uuid = params[:value]
               serie_data
             else
               serie_query
             end

    respond_to do |format|
      format.js
      format.json { render json: @serie }
    end
  end

  # this is and adaptation used by the Heat Map widget
  def get_tops_for_client_latlong

    #params correction when the creation of the widget
    #TODO: modified the defaults  params of the widgets instead of this workaround
    if params[:aggregation] == "bps"
      params[:aggregation] = "clients"
      params[:start_time] = (Time.at(params[:end_time].to_i) - (60*5)).to_i.to_s
    end

    tab = assign_param(@source::COLUMNS.keys, params[:id].to_sym)
    @serie =  @source.where(
      query_type: :group_by,
      range: @range,
      start_time: Time.at(params[:start_time].to_i),
      end_time: Time.at(params[:end_time].to_i),
      granularity: :all,
      group_by: [:client_latlong],
      search: @filter.get,
      limit: params[:limit],
      aggregation: params[:aggregation]
    )

    respond_to do |format|
      format.json {
        render json: @serie.map do|s|
          { tab.to_sym => s.read_attribute(tab),
            @aggregation.to_sym => s.read_attribute(@aggregation),
            secondary_name:
              if @source::COLUMNS[tab.to_sym][:complementary_columns] && tab.to_sym != :sensor_name
                s.send(@source::COLUMNS[tab.to_sym][:complementary_columns].keys.first.to_s)
              else
                s.read_attribute(tab)
              end
          }
        end
      }
    end
  end

  def get_tops
    if params[:id].to_sym == :client_latlong
      get_tops_for_client_latlong
    else
      @tab = params[:id] == 'dot11_status' ? :status : params[:id].split('_').first.to_sym
      @serie = tops_data
      respond_to do |format|
        format.json do
          render json: {
            data: @serie,
            locations: @locations
          }
        end
      end
    end
  end

  private

  def set_saved_location_filters
    @saved_filters = SavedLocationFilter.where(user_id: current_user.id)
    @saved_filters_public = SavedLocationFilter.where
                                               .not(user_id: current_user.id)
                                               .select(&:public)
  end
end
