#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module LocationHelper
  def view_selector_dropdown(current)
    content_tag(:ul, class: 'dropdown-menu') do
      (["Tops", "Map", "Movements", "Sessions"] - [current]).each do |view|
        concat content_tag(:li, link_to(view.to_s, { controller: :location,
                                                action: view.to_s.downcase.to_sym},
                                        class: 'block-ui-but'))
      end
    end
  end

  #
  # Helper to convert minutes to X Hours, Y Minutes format
  #
  def parse_minutes(minutes)
    min = (minutes % 60).to_i
    hours = (minutes / 60).to_i
    result = ''
    result += "#{hours} Hours." if hours > 0
    result += " #{min} Mins." if min > 0
    result
  end

  #
  # Helper to sum all values for certain aggregation at data array
  #
  def total_aggregation_value(data, aggregation)
    data.reduce(0) { |sum, x| sum + x.send(Location::AGGREGATIONS[aggregation][:value]) }.round
  end

  #
  # Helper to set Tooltip Text for Location Custom Button
  #
  def custom_tooltip(range, start_time, end_time)
    return unless (start_time && end_time) && range.to_s == 'custom'
    "From: #{@start_time_for_slider.strftime('%Y/%m/%d %H:%M:%S')} <br>" \
    "To: #{@end_time_for_slider.strftime('%Y/%m/%d %H:%M:%S')}"
  end

  # Helper to saturate a num to a limit
  def saturate(num, limit)
    num > limit ? limit : num
  end

  def location_cast_num(count, aggregation)
    unit = Location::AGGREGATIONS[aggregation][:unit]
    case count
    when 1e12..Float::INFINITY
      "#{(count.to_f / 1e12).round(2)} T#{unit}"
    when 1e9..1e12
      "#{(count.to_f / 1e9).round(2)} G#{unit}"
    when 1e6..1e9
      "#{(count.to_f / 1e6).round(2)} M#{unit}"
    when 1e3..1e6
      "#{(count.to_f / 1e3).round(2)} K#{unit}"
    else
      "#{count} #{unit}"
    end
  end
end
