require 'test_helper'

class LocationControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!
  include Devise::TestHelpers

  context 'super admin user' do
    setup do
      @admin = users(:admin)
      @root = sensors(:root)
      sign_in @admin
    end

    should 'visit Tops view' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :tops
      assert session[:location][:view], 'tops'
    end

    should 'visit Movements view' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :tops
      assert session[:location][:view], 'movements'
    end

    should 'get raw serie for visitors' do
      Redborder::Rbdruid.expects(:query).with do |*args|
        args[0][:query_type] == :timeseries &&
          args[0][:granularity] == :pt5m &&
          args[0][:aggregation] == 'clients'
      end.returns([]).at_least_once

      get :get_serie, id: 'raw', granularity: 'pt5m', aggregation: 'clients', w: 'true',
                      start_time: (Time.now.to_i - 1.hour).to_s, end_time: Time.now.to_i.to_s,
                      format: 'json'
      assert_response :success
    end

    should 'get total of visitors' do
      Redborder::Rbdruid.expects(:query).with do |*args|
        args[0][:query_type] == :timeseries &&
          args[0][:granularity] == :all &&
          args[0][:aggregation] == 'clients'
      end.returns([]).at_least_once

      get :get_serie, id: 'raw', granularity: 'all', aggregation: 'clients', w: 'true',
                      start_time: (Time.now.to_i - 1.hour).to_s, end_time: Time.now.to_i.to_s,
                      format: 'json'
      assert_response :success
    end
  end
end
