#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

  module Jobs

    class RbVulnerabilitiesUpdateDBJob < Struct.new(:user_id, :verbose)

      include Redborder::Jobs::JobHelper
      attr_accessor :content

      # def my_log(str)
      #   File.open("/tmp/lblanco/logs.txt", 'w') do |file|
      #     file.puts str
      #   end
      # end

      def perform #A method "def" can serve as a "begin" statement
        # my_log("#{self.class} has been created with: #{user_id}") if verbose
        content = ("Node executing this job: " + `hostname`.chomp! + "\n\n")
        content += `/usr/lib/redborder/bin/rb_manager_utils.sh -e -n 0 -s /usr/lib/redborder/scripts/rb_vulnerability_load_cvedb.rb`
        #Check if cvedb has been created
        response = `/usr/lib/redborder/bin/rb_manager_utils.sh -e -n 0 -s "[ -f /tmp/rb_vulnerability_load_cvedb_last_update ] && printf true || printf false"`
        # my_log("Checking last update: #{response}") if verbose
        check_last_update = "true" == response
        user_id = User.first.id if user_id.nil? rescue nil
        # my_log("User id: #{user_id}") if verbose
        is_job_ready = check_last_update && !user_id.nil?
        # my_log("Check last update: #{check_last_update}. Is job ready: #{is_job_ready}") if verbose
      rescue Exception => e
        # my_log("rb_vulnerability_load_cvedb has not being previously launched") if verbose
        is_job_ready = false
        content += "#{e.class}: #{e.message}\n"
        e.backtrace.each do |line|
          content += "#{line}\n"
        end
        logit "#{e.class}: #{e.message}" if verbose
      ensure
        # my_log("Creating StoredDelayedJob RbVulnerabilityUpdateDBJob with user_id: #{user_id}")
        StoredDelayedJob.create(job: "RbVulnerabilitiesUpdateDBJob", user_id: user_id, content: content)
        Delayed::Job.enqueue(Redborder::Jobs::RbVulnerabilitiesUpdateDBJob.new(user_id, true), priority: 10, run_at: run_at(is_job_ready))
      end

      def run_at(is_job_ready)
        # Returns the time to run the next instance of the job
        # my_log("Is job ready? #{is_job_ready}")
        # my_log("Periodicity? #{Setting.vulnerability_update_periodicity}")
        if !is_job_ready
          # In case the system is not ready, retry after 10 min
          # my_log("#{self.class} has been renqued because the system is not ready") #if verbose
          Time.now + 10.minutes
          # Just for debugging, the next line:
          # Time.now + 1.minutes
        elsif Setting.vulnerability_update_periodicity == "weekly"
          hour, minute = Setting.vulnerability_update_time.split(":").map(&:to_i)
          1.week.from_now.beginning_of_day + hour.hour + minute.minute
        elsif Setting.vulnerability_update_periodicity == "daily"
          hour, minute = Setting.vulnerability_update_time.split(":").map(&:to_i)
          1.day.from_now.beginning_of_day + hour.hour + minute.minute
        else
          1.hour.from_now
        end
      end
    end
  end
end
